mata:
real scalar rnd_int(real scalar m)
{
// get rand integer in 1,...,m
rnd_perm = jumble(1::m)
first = rnd_perm[1,1]
return(first)
}

mata:
void strata_ind()
{
// create strata categ var (data should be sorted on strata vars)
	external real matrix str_ind
	strlist = st_global("str_var")
	strvars = st_data(.,tokens(strlist))
	n_strvars = cols(strvars)
	fastest = strvars[.,n_strvars]
	curr_fastest = fastest[1,1] 
	str_ind = fastest   // just to create the array	
 	m = 1
	for (r=1; r <= rows(fastest); r++)  {
		if (fastest[r,1] != curr_fastest) {	  // next stratum, step m
			m = m+1
			curr_fastest = fastest[r,1]  
			}
		str_ind[r,1] = m
		}
}

mata:
void do_cens()
// get set for cens within stata, then call routine to do it
// check for relevant sort in calling routine 
// this gets _d and dataset from current sort
{
	external real matrix dataset_str , d_str , str_ind
	vlist = st_global("V_all")	
	strata_ind() 
	n_events = rows(str_ind) 
	n_strata = str_ind[n_events,1]
	for (s=1; s <= n_strata ; s++)  {
		dataset_str = st_data(.,tokens(vlist),str_ind == s)	
		d_str = st_data(.,"_d",str_ind == s)
		do_cens_str()
	}
}

mata:
void do_cens_str()
{
	external real matrix dataset_str, d_str
	n_str = rows(d_str) 
	for (r=1 ; r <= rows(d_str) ; r++) {    // work through stratum
		if (d_str[r,1] == 0) {				// need to censor indiv r
			n_risk = n_str - (r-1)
			rnd_indx = rnd_int(n_risk)+(r-1)	// rnd int in r,...,n_str
			newdat = dataset_str 	// insert sel indiv at position r
			if (rnd_indx > r) {
				row_sel = newdat[rnd_indx,.]
				newdat[(r+1)::rnd_indx,.] = newdat[r::(rnd_indx-1),.]
				newdat[r,.] = row_sel
			}
		}
	}
}

mata:
void init_accum() 
	{
	external real matrix hat_sum, til_sum, drop_hat_sum 
	external real matrix hat_hat_ssq,  til_til_ssq, til_hat_ssq
	external real scalar drop_sum
	b = st_matrix("beta_hat_data")    // just to get dimension
	hat_sum = til_sum = drop_hat_sum = 0 * b							   

	hat_hat_ssq = til_til_ssq = til_hat_ssq = 0 * b * b'
	drop_sum = 0
	}
end

mata :  
void do_accum()
	{
	external real matrix hat_sum, til_sum, drop_hat_sum
	external real matrix hat_hat_ssq, til_til_ssq, til_hat_ssq
	external real scalar drop_sum 

	external real matrix score, score2
	external real scalar llik, llik2
	hat_sum = hat_sum + score
	til_sum = til_sum + score2
	hat_hat_ssq = hat_hat_ssq + score *  score'
	til_til_ssq = til_til_ssq + score2 *  score2'
	til_hat_ssq = til_hat_ssq + score2 * score'
	drop_sum = drop_sum + (llik - llik2)
	drop_hat_sum = drop_hat_sum + (llik - llik2) * score
	}
end

mata:
	void do_score_calcs() 
	// checked results with scorecalc.ado on 3 feb
	// those from scorecalc were checked with Matlab earlier
	// have changed now to ordinary time-sort
	{
	external real scalar llik, llik2
	external real matrix score, score2, X , str_ind

	// X is for current sort but need to get d and str_ind
	d = st_data(.,"_d")
	strata_ind()

	bhat_dta = st_matrix("beta_hat_data")
	btil_dta = st_matrix("beta_til_data")
	denom = denom2 = 0
	llik = llik2 = 0
	numer = numer2 = 0 * bhat_dta
	score = score2 = numer
	n_obs = rows(X)
	for (r=n_obs; r>=1 ; r--)  {	  // working backwards
		z = X[r,.]'
		expon = exp(z' * bhat_dta)
        // re-initialize when stratum changes
        if (r<n_obs) {
          rp1 = r+1 
          if (str_ind[r,1] != str_ind[rp1,1]) {
            numer = numer * 0 
	        denom = 0
	      }
        }
		numer = numer + z * expon
		denom = denom + expon
		Ez = numer / denom
		score = score + d[r,1] * (z - Ez)
		llik = llik + d[r,1] * (z' * bhat_dta - log(denom)) 
	
		// again for hyp fit
		expon = exp(z' * btil_dta)
        if (r<n_obs) {
          if (str_ind[r,1] != str_ind[rp1,1]) {
            numer2 = numer2 * 0 
	        denom2 = 0
	      }
        }
		numer2 = numer2 + z * expon
		denom2 = denom2 + expon
		Ez2 = numer2 / denom2
		score2 = score2 + d[r,1] * (z - Ez2)
		llik2 = llik2 + d[r,1] * (z' * btil_dta - log(denom2)) 
		}  
	}
end 

mata: 
	void get_X()
	{
	external real matrix X
	external scalar nval
	vlist = st_global("V_all")	
	X = st_data(.,tokens(vlist))	  // way to make matrix from dataset cols $V_all
	nval = rows(X)
	}
end

mata:
	void rstar_calc()
	{
	external real scalar nval, drop_sum, NP_adj, INF_adj, rstar, rval_data
	external real scalar Pval_Sk, Pval_r
	external real matrix hat_sum, til_sum, drop_hat_sum
	external real matrix hat_hat_ssq, til_til_ssq, til_hat_ssq
	p = rows(hat_hat_ssq)
	nskov = st_numscalar("nskov")
	info_hat = st_matrix("vari_hat_data")
	info_hat = invsym(info_hat)
	info_hyp = st_matrix("vari_til_data")
	info_hyp = invsym(info_hyp) 
	cov_Uhat_Uhat = hat_hat_ssq -  hat_sum * hat_sum' / nskov
	cov_Util_Util = til_til_ssq -  til_sum * til_sum' / nskov
	cov_Util_Uhat = til_hat_ssq -  til_sum * hat_sum' / nskov
cov_Util_Uhat = cov_Util_Uhat'	// probably changing to skovgaard form
								// as opposed to severini one
	cov_del_Uhat = 	drop_hat_sum - hat_sum * drop_sum /  nskov

	einfo_sim = makesymmetric(cov_Uhat_Uhat) ;
	el_th_thhat = cov_Util_Uhat * invsym(einfo_sim) * info_hat 
	del_el_thhat = cov_del_Uhat' * invsym(einfo_sim) * info_hat
	el_nu_thhat = el_th_thhat[2..p , .]
	el_nu_nu = el_th_thhat[2..p , 2..p]
	elprof_psihat = del_el_thhat[1,1] - el_nu_thhat[.,1]' * invsym(el_nu_nu) * del_el_thhat[2..p]'
	info_hat_inv = invsym(info_hat)
	info_prof = 1 / info_hat_inv[1,1]
	util = elprof_psihat/sqrt(info_prof) ;	
	C = det(el_nu_nu) / sqrt(det(info_hyp) * det(info_hat[2..p,2..p])) ;
	rval_data  = st_numscalar("rval_data")
	if (abs(rval_data) > .001) {
		NP_adj = (1/rval_data) * log(C) 		 ;
		INF_adj = (1/rval_data) * log(util/rval_data)  ;
		}
	else {
		NP_adj = 0 ;
		INF_adj = 0 ;
		}
	rstar = rval_data + NP_adj + INF_adj
	Pval_Sk = normal(rstar)
	Pval_r = normal(rval_data)
	}
end

mata:
	void Pval_calc()
	{
	external real scalar Pval, Pval_Sk, Pval_r
	external real scalar Pval_Sk  , NP_adj , INF_adj 
	rvals = st_data(., "rvals")
	rval_data = st_numscalar("rval_data")
	rstar_FRS = st_numscalar("rstar_FRS")
	NP_frs = st_numscalar("NP_frs")
	INF_frs = st_numscalar("INF_frs")
	noPvalsim = st_numscalar("noPvalsim")
	Pval_r = normal(rval_data)
	Pval_FRS = normal(rstar_FRS)
	if (noPvalsim==0) {
		lesser = rvals :< rval_data
		ntrial = rows(rvals)
		Pval_sim = mean(lesser,1)  // mean wgts 1
		}
	else {
		Pval_sim = 0
		}
	if (Pval > 0.5) {
		Pval_Sk = 1 - Pval_Sk
		Pval_r = 1 - Pval_r
		Pval_FRS = 1 -Pval_FRS
		Pval_sim = 1 - Pval_sim
		}
	printf("rvalue_observed = %5.4f\n", rval_data)
	printf("Pval_r = %5.4f\n\n", Pval_r)
	if (noPvalsim==0) {
		printf("Pval_condl_BS = %06.4f", Pval_sim)
		SE = sqrt(Pval_sim * (1 - Pval_sim) / ntrial)
		printf("     Std Error = %06.4f\n\n" , SE)
		}
	printf("Pval_condl__2nd_order = %5.4f\n", Pval_Sk)
	printf("    NP adj = %5.4f" , NP_adj)
	printf("  INF adj = %5.4f\n\n" , INF_adj)
	printf("Pval_FRS_2nd_order = %5.4f\n", Pval_FRS)
	printf("    NP adj = %5.4f" , NP_frs)
	printf("  INF adj = %5.4f\n" , INF_frs)
	}
end 



